<?php
define("CAPTCHAFA_API_SERVER", "http://www.captchafa.com/api");
define("CAPTCHAFA_VERIFY_SERVER", "www.captchafa.com");
error_reporting(0);
/**
 * Encodes the given data into a query string format
 * @param $data - array of string elements to be encoded
 * @return string - encoded request
 */
function _captchafa_qsencode ($data) {
        $req = "";
        foreach ( $data as $key => $value )
                $req .= $key . '=' . urlencode( stripslashes($value) ) . '&';

        // Cut the last '&'
        $req=substr($req,0,strlen($req)-1);
        return $req;
}



/**
 * Submits an HTTP POST to a CAPTCHAfa server
 * @param string $host
 * @param string $path
 * @param array $data
 * @param int port
 * @return array response
 */
function _captchafa_http_post($host, $path, $data, $port = 80) {

        $req = _captchafa_qsencode ($data);

        $http_request  = "POST $path HTTP/1.0\r\n";
        $http_request .= "Host: $host\r\n";
        $http_request .= "Content-Type: application/x-www-form-urlencoded;\r\n";
        $http_request .= "Content-Length: " . strlen($req) . "\r\n";
        $http_request .= "User-Agent: CAPTCHAfa/PHP\r\n";
        $http_request .= "\r\n";
        $http_request .= $req;

        $response = '';
        if( false == ( $fs = @fsockopen($host, $port, $errno, $errstr, 10) ) ) {
                echo $errstr;
				die ('Could not open socket');
        }

        fwrite($fs, $http_request);

        while ( !feof($fs) )
                $response .= fgets($fs, 1160); // One TCP-IP packet
        fclose($fs);
        $response = explode("\r\n\r\n", $response, 2);

        return $response;
}



/**
 * Gets the challenge HTML (javascript and non-javascript version).
 * This is called from the browser, and the resulting CAPTCHAfa HTML widget
 * is embedded within the HTML form it was called from.
 * @param string $pubkey A public key for CAPTCHAfa
 * @param string $error The error given by CAPTCHAfa (optional, default is null)

 * @return string - The HTML to be embedded in the user's form.
 */
function captchafa_get_html ($pubkey, $error = null)
{
	if ($pubkey == null || $pubkey == '') {
		die ("To use CAPTCHAfa you must get an API key from <a href='http://www.captchafa.com/getkey/'>http://www.captchafa.com/getkey/</a>");
	}
	
	$server = CAPTCHAFA_API_SERVER;
        $errorpart = "";
        if ($error) {
           $errorpart = "&amp;error=" . $error;
        }
        return '<script type="text/javascript" src="'. $server . '/?challenge&k=' . $pubkey . $errorpart . '"></script>';
}




/**
 * A CaptchaFaResponse is returned from captchafa_check_answer()
 */
class CaptchaFaResponse {
        var $is_valid;
        var $error;
}


/**
  * Calls an HTTP POST function to verify if the user's guess was correct
  * @param string $privkey
  * @param string $remoteip
  * @param string $challenge
  * @param string $response
  * @param array $extra_params an array of extra variables to post to the server
  * @return CaptchaFaResponse
  */
function captchafa_check_answer ($privkey, $remoteip, $challenge, $response, $extra_params = array())
{
	if ($privkey == null || $privkey == '') {
		die ("To use CAPTCHAfa you must get an API key from <a href='http://www.captchafa.com/getkey/'>http://www.captchafa.com/getkey/</a>");
	}

	if ($remoteip == null || $remoteip == '') {
		die ("For security reasons, you must pass the remote ip to CAPTCHAfa");
	}

        //discard spam submissions
        if ($challenge == null || strlen($challenge) == 0 || $response == null || strlen($response) == 0) {
				echo strlen($challenge);
                $captchafa_response = new CaptchaFaResponse();
                $captchafa_response->is_valid = false;
                $captchafa_response->error = 'incorrect-captcha-sol';
                return $captchafa_response;
        }

        $response = _captchafa_http_post (CAPTCHAFA_VERIFY_SERVER, "/api/verify/",
                                          array (
                                                 'privatekey' => $privkey,
                                                 'remoteip' => $remoteip,
                                                 'challenge' => $challenge,
                                                 'response' => $response
                                                 ) + $extra_params
                                          );
		$answers = explode ("\n", $response [1]);
        $captchafa_response = new CaptchaFaResponse();
        if (trim ($answers [0]) == 'true') {
                $captchafa_response->is_valid = true;
        }
        else {
                $captchafa_response->is_valid = false;
                $captchafa_response->error = $answers [1];
        }
        return $captchafa_response;
}

/**
 * gets a URL where the user can sign up for CAPTCHAfa. If your application
 * has a configuration page where you enter a key, you should provide a link
 * using this function.
 * @param string $domain The domain where the page is hosted
 * @param string $appname The name of your application
 */
function captchafa_get_signup_url ($domain = null, $appname = null) {
	return "http://www.captchafa.com/getkey/?" .  _captchafa_qsencode (array ('domains' => $domain, 'app' => $appname));
}

function _captchafa_aes_pad($val) {
	$block_size = 16;
	$numpad = $block_size - (strlen ($val) % $block_size);
	return str_pad($val, strlen ($val) + $numpad, chr($numpad));
}
?>
